@@ -26,6 +26,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#ifdef HAVE_LOGINCAP
+#include <login_cap.h>
+#endif
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -124,7 +127,6 @@ extern gint GdmRetryDelay;
 extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gchar *GdmGlobalFaceDir;
-extern gboolean GdmBrowser;
 extern gboolean GdmDebug;
 
 
@@ -539,7 +541,7 @@ gdm_slave_run (GdmDisplay *display)
 	    /* this usually doesn't return */
 	    gdm_slave_chooser ();  /* Run the chooser */
 	    return;
-    } else if ((d->console || GdmAllowRemoteAutoLogin) &&
+    } else if (d->type == TYPE_LOCAL &&
 	       gdm_first_login &&
 	       ! ve_string_empty (ParsedAutomaticLogin) &&
 	       strcmp (ParsedAutomaticLogin, "root") != 0) {
@@ -1014,31 +1016,15 @@ gdm_slave_wait_for_login (void)
 }
 
 /* If path starts with a "trusted" directory, don't sanity check things */
+/* This is really somewhat "outdated" as we now really want things in
+ * the picture dir or in ~/.gnome/photo */
 static gboolean
 is_in_trusted_pic_dir (const char *path)
 {
-	GSList *locations = NULL, *li;
-
 	/* our own pixmap dir is trusted */
 	if (strncmp (path, EXPANDED_PIXMAPDIR, sizeof (EXPANDED_PIXMAPDIR)) == 0)
 		return TRUE;
 
-	g_free (gnome_program_locate_file (NULL /* program */,
-					   GNOME_FILE_DOMAIN_PIXMAP,
-					   "" /* file_name */,
-					   FALSE /* only_if_exists */,
-					   &locations));
-	for (li = locations; li != NULL; li = li->next) {
-		/* gnome's pixmap dirs are trusted */
-		if (strncmp (path, li->data, strlen (li->data)) == 0) {
-			g_slist_foreach (locations, (GFunc)g_free, NULL);
-			g_slist_free (locations);
-			return TRUE;
-		}
-	}
-	g_slist_foreach (locations, (GFunc)g_free, NULL);
-	g_slist_free (locations);
-
 	return FALSE;
 }
 
@@ -1047,8 +1033,6 @@ static void
 run_pictures (void)
 {
 	char *response;
-	int tempfd;
-	char tempname[256];
 	char buf[1024];
 	size_t bytes;
 	struct passwd *pwent;
@@ -1058,19 +1042,16 @@ run_pictures (void)
 
 	response = NULL;
 	for (;;) {
+		struct stat s;
+		char *tmp, *ret;
+		int i;
+
 		g_free (response);
 		response = gdm_slave_greeter_ctl (GDM_NEEDPIC, "");
 		if (ve_string_empty (response)) {
 			g_free (response);
 			return;
 		}
-		
-		/* don't quit, we don't want to further confuse a confused
-		 * greeter, just don't send it pics */
-		if ( ! GdmBrowser) {
-			gdm_slave_greeter_ctl_no_ret (GDM_READPIC, "");
-			continue;
-		}
 
 		pwent = getpwnam (response);
 		if (pwent == NULL) {
@@ -1193,6 +1174,14 @@ run_pictures (void)
 			picfile = g_strconcat (pwent->pw_dir, "/.gnome/photo", NULL);
 		}
 
+		if (stat (picfile, &s) < 0) {
+			seteuid (0);
+			setegid (GdmGroupId);
+
+			gdm_slave_greeter_ctl_no_ret (GDM_READPIC, "");
+			continue;
+		}
+
 		fp = fopen (picfile, "r");
 		g_free (picfile);
 		if (fp == NULL) {
@@ -1203,32 +1192,37 @@ run_pictures (void)
 			continue;
 		}
 
-		strcpy (tempname, "/tmp/gdm-user-picture-XXXXXX");
-		tempfd = g_mkstemp (tempname);
-
-		if (tempfd < 0) {
-			fclose (fp);
+		tmp = g_strdup_printf ("buffer:%d", (int)s.st_size);
+		ret = gdm_slave_greeter_ctl (GDM_READPIC, tmp);
+		g_free (tmp);
 
+		if (ret == NULL || strcmp (ret, "OK") != 0) {
+			g_free (ret);
 			seteuid (0);
 			setegid (GdmGroupId);
-
-			gdm_slave_greeter_ctl_no_ret (GDM_READPIC, "");
 			continue;
 		}
+		g_free (ret);
 
-		fchmod (tempfd, 0644);
+		g_print ("%c", STX);
 
+		i = 0;
 		while ((bytes = fread (buf, sizeof (char),
 				       sizeof (buf), fp)) > 0) {
-			write (tempfd, buf, bytes);
+			fwrite (buf, 1, bytes, stdout);
+			i += bytes;
 		}
 
 		fclose (fp);
-		close (tempfd);
 
-		gdm_slave_greeter_ctl_no_ret (GDM_READPIC, tempname);
-
-		unlink (tempname);
+		/* eek, this "could" happen, so just send some garbage */
+		while (i < s.st_size) {
+			bytes = MIN (sizeof (buf), s.st_size - i);
+			fwrite (buf, 1, bytes, stdout);
+			i += bytes;
+		}
+			
+		gdm_slave_greeter_ctl_no_ret (GDM_READPIC, "done");
 
 		seteuid (0);
 		setegid (GdmGroupId);
@@ -2005,9 +1999,11 @@ dequote (const char *in)
 static void
 session_child_run (struct passwd *pwent,
 		   const char *home_dir,
+		   gboolean home_dir_ok,
 		   const char *session,
 		   const char *save_session,
 		   const char *language,
+		   gboolean def_language,
 		   const char *gnome_session,
 		   gboolean usrcfgok,
 		   gboolean savesess,
@@ -2016,13 +2012,29 @@ session_child_run (struct passwd *pwent,
 		   gboolean savegnomesess)
 {
 	sigset_t mask; 
+	int logfd;
 	int i;
+	gboolean failsafe = FALSE;
 	char *sesspath, *sessexec;
 	gboolean need_config_sync = FALSE;
 	const char *shell = NULL;
+	Display *disp;
+
+	disp = XOpenDisplay (d->name);
+	if (disp != NULL) {
+		XSetInputFocus (disp, PointerRoot,
+				RevertToPointerRoot, CurrentTime);
+		XCloseDisplay (disp);
+	}
+
 
 	gdm_clearenv ();
 
+	if (setsid() < 0)
+		/* should never happen */
+		gdm_error (_("%s: setsid() failed: %s!"),
+			   "session_child_run", strerror(errno));
+
 	/* Prepare user session */
 	gnome_setenv ("XAUTHORITY", d->userauth, TRUE);
 	gnome_setenv ("DISPLAY", d->name, TRUE);
@@ -2050,10 +2062,6 @@ session_child_run (struct passwd *pwent,
 		language = unaliaslang (language);
 	}
 
-	/* Set locale */
-	gnome_setenv ("LANG", language, TRUE);
-	gnome_setenv ("GDM_LANG", language, TRUE);
-    
 	setpgid (0, 0);
 	
 	umask (022);
@@ -2069,9 +2077,34 @@ session_child_run (struct passwd *pwent,
 	 * not to leave the egid around */
 	setegid (pwent->pw_gid);
 
+#ifdef HAVE_LOGINCAP
+	if (setusercontext (NULL, pwent, pwent->pw_uid,
+			    LOGIN_SETLOGIN | LOGIN_SETPATH |
+			    LOGIN_SETPRIORITY | LOGIN_SETRESOURCES |
+			    LOGIN_SETUMASK | LOGIN_SETUSER) < 0)
+		gdm_child_exit (DISPLAY_REMANAGE,
+				_("%s: setusercontext() failed for %s. "
+				  "Aborting."), "gdm_slave_session_start",
+				login);
+
+	/* A different language was selected, or taken from the saved
+	 * prefs of the user */
+	if ( ! def_language) {
+		gnome_setenv ("LANG", language, TRUE);
+		gnome_setenv ("GDM_LANG", language, TRUE);
+	} else {
+		/* setusercontext sets up user languages */
+		gnome_setenv ("GDM_LANG", g_getenv ("LANG"), TRUE);
+	}
+#else
 	if (setuid (pwent->pw_uid) < 0) 
 		gdm_child_exit (DISPLAY_REMANAGE,
 				_("gdm_slave_session_start: Could not become %s. Aborting."), login);
+
+	/* Set locale */
+	gnome_setenv ("LANG", language, TRUE);
+	gnome_setenv ("GDM_LANG", language, TRUE);
+#endif
 	
 	chdir (home_dir);
 
@@ -2079,7 +2112,7 @@ session_child_run (struct passwd *pwent,
 	 * to write */
 	gnome_config_drop_all ();
 	
-	if (usrcfgok && savesess) {
+	if (usrcfgok && savesess && home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir,
 					     "/.gnome/gdm=/session/last", NULL);
 		gnome_config_set_string (cfgstr, save_session);
@@ -2087,7 +2120,7 @@ session_child_run (struct passwd *pwent,
 		g_free (cfgstr);
 	}
 	
-	if (usrcfgok && savelang) {
+	if (usrcfgok && savelang && home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir,
 					     "/.gnome/gdm=/session/lang", NULL);
 		gnome_config_set_string (cfgstr, language);
@@ -2097,7 +2130,8 @@ session_child_run (struct passwd *pwent,
 
 	if (sessoptok &&
 	    savegnomesess &&
-	    gnome_session != NULL) {
+	    gnome_session != NULL &&
+	    home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir, "/.gnome/session-options=/Options/CurrentSession", NULL);
 		gnome_config_set_string (cfgstr, gnome_session);
 		need_config_sync = TRUE;
@@ -2164,6 +2198,7 @@ session_child_run (struct passwd *pwent,
 				   "run.  This is only to fix problems in\n"
 				   "your installation."));
 		}
+		failsafe = TRUE;
 	}
 
 	/* an if and not an else, we could have done a fall-through
@@ -2192,8 +2227,14 @@ session_child_run (struct passwd *pwent,
 				   "'exit' and an enter into the window."));
 			focus_first_x_window ("xterm");
 		}
+		failsafe = TRUE;
 	} 
 
+	/* hack */
+	if (strcmp (session, "Failsafe") == 0) {
+		failsafe = TRUE;
+	}
+
 	if (sesspath == NULL) {
 		if (GdmSessDir != NULL) {
 			sesspath = g_strconcat
@@ -2215,6 +2256,21 @@ session_child_run (struct passwd *pwent,
 		shell = "/bin/sh";
 	}
 
+        /* Log all output from session programs to a file,
+	 * unless in failsafe mode which needs to work when there is
+	 * no diskspace as well */
+	if ( ! failsafe && home_dir_ok) {
+		logfd = open (g_strconcat (home_dir, "/.xsession-errors", NULL),
+			      O_CREAT|O_TRUNC|O_WRONLY, 0644);
+		if (logfd != -1) {
+			dup2 (logfd, 1);
+			dup2 (logfd, 2);
+		} else {
+			gdm_error (_("%s: Could not open ~/.xsession-errors"),
+				   "run_session_child");
+		}
+	}
+
 	/* just a stupid test, the below would fail, but this gives a better
 	 * message */
 	if (strcmp (shell, "/sbin/nologin") == 0 ||
@@ -2259,8 +2315,11 @@ gdm_slave_session_start (void)
     char *gnome_session = NULL;
     gboolean savesess = FALSE, savelang = FALSE, savegnomesess = FALSE;
     gboolean usrcfgok = FALSE, sessoptok = FALSE, authok = FALSE;
+    gboolean def_language = FALSE;
     const char *home_dir = NULL;
     gboolean home_dir_ok = FALSE;
+    time_t session_start_time, end_time; 
+    gboolean failsafe = FALSE;
     pid_t pid;
 
     gdm_debug ("gdm_slave_session_start: Attempting session for user '%s'",
@@ -2392,12 +2451,13 @@ gdm_slave_session_start (void)
 		    language = g_strdup (lang);
 	    else
 		    language = g_strdup (GdmDefaultLocale);
-	    savelang = TRUE;
 
 	    if (ve_string_empty (language)) {
 		    g_free (language);
 		    language = g_strdup ("C");
 	    }
+
+	    def_language = TRUE;
     }
 
     /* save this session as the users session */
@@ -2469,15 +2529,36 @@ gdm_slave_session_start (void)
     setegid (GdmGroupId);
     
     if ( ! authok) {
-	gdm_debug ("gdm_slave_session_start: Auth not OK");
-	gdm_slave_session_stop (0);
-	gdm_slave_session_cleanup ();
-	
-	gdm_server_stop (d);
-	gdm_verify_cleanup (d);
+	    gdm_debug ("gdm_slave_session_start: Auth not OK");
 
-	_exit (DISPLAY_REMANAGE);
-    } 
+	    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+
+	    gdm_error_box (d,
+			   GTK_MESSAGE_ERROR,
+			   _("GDM could not write to your authorization\n"
+			     "file.  This could mean that you are out of\n"
+			     "disk space or that your home directory could\n"
+			     "not be opened for writing.  In any case, it\n"
+			     "is not possible to log in.  Please contact\n"
+			     "your system administrator"));
+
+	    gdm_slave_session_stop (0);
+	    gdm_slave_session_cleanup ();
+
+	    gdm_server_stop (d);
+	    gdm_verify_cleanup (d);
+
+	    _exit (DISPLAY_REMANAGE);
+    }
+
+    if (strcmp (session, GDM_SESSION_FAILSAFE_GNOME) == 0 ||
+	strcmp (session, GDM_SESSION_FAILSAFE_XTERM) == 0 ||
+	strcmp (session, "Failsafe") == 0 /* hack */)
+	    failsafe = TRUE;
+
+    /* don't completely rely on this, the user
+     * could reset time or do other crazy things */
+    session_start_time = time (NULL);
 
     /* Start user process */
     gdm_sigchld_block_push ();
@@ -2496,9 +2577,11 @@ gdm_slave_session_start (void)
 	/* Never returns */
 	session_child_run (pwent,
 			   home_dir,
+			   home_dir_ok,
 			   session,
 			   save_session,
 			   language,
+			   def_language,
 			   gnome_session,
 			   usrcfgok,
 			   savesess,
@@ -2531,6 +2614,36 @@ gdm_slave_session_start (void)
 	    gdm_sigchld_block_pop ();
     }
 
+    end_time = time (NULL);
+
+    gdm_debug ("Session: start_time: %ld end_time: %ld",
+	       (long)session_start_time, (long)end_time);
+
+    if  ((/* sanity */ end_time >= session_start_time) &&
+	 (end_time - 10 <= session_start_time)) {
+	    char *errfile = g_strconcat (home_dir, "/.xsession-errors", NULL);
+	    gdm_debug ("Session less then 10 seconds!");
+
+	    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+
+	    /* FIXME: perhaps do some checking to display a better error,
+	     * such as gnome-session missing and such things. */
+	    gdm_error_box_full (d,
+				GTK_MESSAGE_WARNING,
+				_("Your session only lasted less then\n"
+				  "10 seconds.  If you have not logged out\n"
+				  "yourself, this could mean that there is\n"
+				  "some installation problem or that you may\n"
+				  "be out of diskspace.  Try logging in with\n"
+				  "one of the failsafe sessions to see if you\n"
+				  "can fix this problem."),
+				(home_dir_ok && ! failsafe) ?
+			       	  _("View details (~/.xsession-errors file)") :
+				  NULL,
+				errfile);
+	    g_free (errfile);
+    }
+
     gdm_debug ("gdm_slave_session_start: Session ended OK");
 
     gdm_slave_session_stop (pid);
